/*****************************************************************************\
 ***************************************************************************** 
 **********   GENERIC REGISTER PROTECTION MACROS AND VERIFICATION   **********
 ***************************************************************************** 
\*****************************************************************************/

/*
   MJR 16-Oct-2007:

   When writing these macros I have included some doxygen like comments into
   the corresponding definition. I hope this is useful information and does 
   help in using them. Please direct any suggestions for improvements to me.

   These macros are designed to be used in conjunction with the definitions
   in the #define based header file. Therefore, whenever they need a register 
   definition they expect a dereferenced pointer value; all macros using this 
   form of a register value use the parameter <thereg> for the register. 
   There is a second set of macros that work with an address value (numerical 
   value); these macros have the special suffix _ADDRESS and use a parameter 
   <reg_addr>. There are currently no macros for usage with structures.
 */

/*****************************************************************************\
 * REGISTER PROTECTION: Level1 Macros - this is the user interface
\*****************************************************************************/

/*! Modify the register content while concurrently setting the soft lock bit.

    A macro of type WRITE_WITH_LOCK<size> with <size> = { 8, 16, 32 } writes
    the given register <thereg> with the new value <newvalue>. Concurrently
    it sets the corresponding soft lock bit(s) assigned to this register.
    The value of size selects the access width for this register, which must
    be one of 8, 16, or 32 bits.

    @PARAM thereg     Pointer to the given register
    @PARAM newvalue   Value to be written to the given register
 */

#define WRITE_WITH_LOCK8(thereg,newvalue) \
   WRITE_WITH_LOCK8_ADDRESS(LOCK__REG2ADDR(thereg),(newvalue))

#define WRITE_WITH_LOCK8_ADDRESS(reg_addr,newvalue) \
   ((*((vuint8_t*)((reg_addr)+LOCK__OFFSET))) = (newvalue))


#define WRITE_WITH_LOCK16(thereg,newvalue) \
   WRITE_WITH_LOCK16_ADDRESS(LOCK__REG2ADDR(thereg),(newvalue))

#define WRITE_WITH_LOCK16_ADDRESS(reg_addr,newvalue) \
   ((*((vuint16_t*)((reg_addr)+LOCK__OFFSET))) = (newvalue))


#define WRITE_WITH_LOCK32(thereg,newvalue) \
   WRITE_WITH_LOCK32_ADDRESS(LOCK__REG2ADDR(thereg),(newvalue))

#define WRITE_WITH_LOCK32_ADDRESS(reg_addr,newvalue) \
   ((*((vuint32_t*)((reg_addr)+LOCK__OFFSET))) = (newvalue))


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *\
   REGISTER PROTECTION: Working with Soft Lock Bits
\* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

/*! Retrieves the soft lock bits assigned to the given register.

    The macro GET_SOFTLOCK determines the soft lock bits assigned to the given
    register specified with <thereg> and assignes them to the variable <dest>; 
    this variable should receive an eight bit value, which is the maximum size
    for the soft lock bits for a single register.

    @PARAM thereg   Pointer to the given register
    @PARAM dest     Variable receiving the soft lock bit information

    @SEE SET_SOFTLOCK8 SET_SOFTLOCK16 SET_SOFTLOCK32 
         CLR_SOFTLOCK8 CLR_SOFTLOCK16 CLR_SOFTLOCK32
 */

// Retrieves the soft lock bits for a given register and stores it into dest
#define GET_SOFTLOCK(thereg,dest) \
   GET_SOFTLOCK_ADDRESS(LOCK__REG2ADDR(thereg),(dest))

#define GET_SOFTLOCK_ADDRESS(reg_addr,dest) \
   ((dest) = (*(LOCK__SLB_REGISTER(reg_addr))))


/*! Sets the soft lock bits assigned to the given register.

    A macro of type SET_SOFTLOCK<size> with <size> = { 8, 16, 32 } sets the
    soft lock bits assigned to the given register <thereg>. The value of size 
    selects the access width (and also the amount of affected bits) for this 
    register, which must be one of 8, 16, or 32 bits.

    @NOTE Soft lock bits can only be written in supervisor mode only 
   
    @PARAM thereg   Pointer to the given register

    @SEE GET_SOFTLOCK CLR_SOFTLOCK8 CLR_SOFTLOCK16 CLR_SOFTLOCK32
 */

// Sets the soft lock bit for the given (8 bit wide) register
#define SET_SOFTLOCK8(thereg) \
   SET_SOFTLOCK8_ADDRESS(LOCK__REG2ADDR(thereg))

#define SET_SOFTLOCK8_ADDRESS(reg_addr) \
   ((*LOCK__SLB_REGISTER(reg_addr)) = LOCK__WRITE_MASK08(reg_addr,LOCK__SET_MASK))


// Sets the soft lock bit for the given (16 bit wide) register
#define SET_SOFTLOCK16(thereg) \
   SET_SOFTLOCK16_ADDRESS(LOCK__REG2ADDR(thereg))

#define SET_SOFTLOCK16_ADDRESS(reg_addr) \
   ((*LOCK__SLB_REGISTER(reg_addr)) = LOCK__WRITE_MASK16(reg_addr,LOCK__SET_MASK))


// Sets the soft lock bit for the given (32 bit wide) register
#define SET_SOFTLOCK32(thereg) \
   SET_SOFTLOCK32_ADDRESS(LOCK__REG2ADDR(thereg))

#define SET_SOFTLOCK32_ADDRESS(reg_addr) \
   ((*LOCK__SLB_REGISTER(reg_addr)) = LOCK__WRITE_MASK32(reg_addr,LOCK__SET_MASK))


/*! Clears the soft lock bits assigned to the given register.

    A macro of type CLR_SOFTLOCK<size> with <size> = { 8, 16, 32 } clears the
    soft lock bits assigned to the given register <thereg>. The value of size 
    selects the access width (and also the amount of affected bits) for this 
    register, which must be one of 8, 16, or 32 bits.

    @NOTE Soft lock bits can only be written in supervisor mode only 
   
    @PARAM thereg   Pointer to the given register

    @SEE GET_SOFTLOCK SET_SOFTLOCK8 SET_SOFTLOCK16 SET_SOFTLOCK32
 */

// Clears the soft lock bit for the given (8 bit wide) register
#define CLR_SOFTLOCK8(thereg) \
   CLR_SOFTLOCK8_ADDRESS(LOCK__REG2ADDR(thereg))

#define CLR_SOFTLOCK8_ADDRESS(reg_addr) \
   ((*LOCK__SLB_REGISTER(reg_addr)) = LOCK__WRITE_MASK08(reg_addr,LOCK__CLR_MASK))


// Clears the soft lock bit for the given (16 bit wide) register
#define CLR_SOFTLOCK16(thereg) \
   CLR_SOFTLOCK16_ADDRESS(LOCK__REG2ADDR(thereg))

#define CLR_SOFTLOCK16_ADDRESS(reg_addr) \
   ((*LOCK__SLB_REGISTER(reg_addr)) = LOCK__WRITE_MASK16(reg_addr,LOCK__CLR_MASK))


// Clears the soft lock bit for the given (32 bit wide) register
#define CLR_SOFTLOCK32(thereg) \
   CLR_SOFTLOCK32_ADDRESS(LOCK__REG2ADDR(thereg))

#define CLR_SOFTLOCK32_ADDRESS(reg_addr) \
   ((*LOCK__SLB_REGISTER(reg_addr)) = LOCK__WRITE_MASK32(reg_addr,LOCK__CLR_MASK))


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *\
   REGISTER PROTECTION: Accessing the Global Control Register (GCR)
\* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

/*! Sets the hardlock bit HLB in the module's Global Control Register GCR.

    The macro SET_HARDLOCK sets the hard lock bit in the module specific Global
    Control Register (GCR). Setting this bit will block the modification of all
    soft lock bits, effectively freezing the actual lock state of the module.
    This bit can only be cleared by a system reset.

    @PARAM base   Base address of the given module

    @NOTE: This macro must be executed in supervisor mode only

    @SEE  USER_ACCESS_FORBIDDEN USER_ACCESS_ALLOWED
 */

#define SET_HARDLOCK(base) \
   ((*LOCK__GCR_REGISTER(base)) = ((*LOCK__GCR_REGISTER(base)) | LOCK__GCR_HLB))

/*! Clears the UAA bit in the module's Global Control Register GCR.

    The macro USER_ACCESS_FORBIDDEN clears the User Access Allowed bit in the
    module's Global Control Register GCR; permitting access only in supervisor
    mode. 

    @PARAM base   Base address of the given module

    @NOTE: This macro must be executed in supervisor mode only

    @SEE  SET_HARDLOCK USER_ACCESS_ALLOWED
 */

#define USER_ACCESS_FORBIDDEN(base) \
   ((*LOCK__GCR_REGISTER(base)) = ((*LOCK__GCR_REGISTER(base)) & (~LOCK__GCR_UAA)))

/*! Sets the UAA bit in the module's Global Control Register GCR.

    The macro USER_ACCESS_ALLOWED sets the User Access Allowed bit in the
    module's Global Control Register GCR; permitting access to the module
    register in any mode defined by the module's protection scheme.

    @PARAM base   Base address of the given module

    @NOTE: This macro must be executed in supervisor mode only

    @SEE  SET_HARDLOCK USER_ACCESS_FORBIDDEN
 */

#define USER_ACCESS_ALLOWED(base) \
   ((*LOCK__GCR_REGISTER(base)) = ((*LOCK__GCR_REGISTER(base)) | LOCK__GCR_UAA))

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *\
   REGISTER PROTECTION: Global Control Register (GCR) field definitions
\* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

#define LOCK__GCR_HLB_VALUE(x)  (((x)&1) << LOCK__GCR_HLB_BIT)
#define LOCK__GCR_HLB_BIT       (31)
#define LOCK__GCR_HLB           (1 << LOCK__GCR_HLB_BIT)

#define LOCK__GCR_UAA_VALUE(x)  (((x)&1) << LOCK__GCR_UAA_BIT)
#define LOCK__GCR_UAA_BIT       (23)
#define LOCK__GCR_UAA           (1 << LOCK__GCR_UAA_BIT)

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *\
   REGISTER PROTECTION: Direct access to register protection gasket registers
\* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

// Provides the content of the SLB register for a given module register; here
//  it is possible to either use the register pointer or the related address.
//  Please note that a SLB register is always 8 bit wide as is the result.
#define LOCK_SLB(thereg) \
   LOCK_SLB_ADDRESS(LOCK__REG2ADDR(thereg))

#define LOCK_SLB_ADDRESS(reg_addr) \
   (*LOCK__SLB_REGISTER(reg_addr))

// Provides the content of the GCR register of the protection register gasket;
//  this macro requires the base address of the corresponding module
#define LOCK_GCR(base) \
   (*LOCK__GCR_REGISTER(base))

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *\
   REGISTER PROTECTION: Helper macros for Level1 macros
\* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

/* 
   !!! CAUTION !!! CAUTION !!! CAUTION !!! CAUTION !!! CAUTION !!! CAUTION !!! 
   Never use any of these macros directly; they may change without notice
   !!! CAUTION !!! CAUTION !!! CAUTION !!! CAUTION !!! CAUTION !!! CAUTION !!! 
 */

// Provides a pointer to the SLB register from a given module register addr; 
//  please note that a SLB register is always 8 bit wide
#define LOCK__SLB_REGISTER(reg_addr) \
   ((vuint8_t*) LOCK__SLB_REG_ADDR(reg_addr))

// Provides a pointer to the GCR register of the protection register gasket;
//  this macro requires the base address of the corresponding module 
#define LOCK__GCR_REGISTER(base) \
   ((vuint32_t*) LOCK__GCR_REG_ADDR(base))


// Determines the address of the given register from the given pointer
#define LOCK__REG2ADDR(thereg) \
   ((uint32_t) &(thereg))


// Mask that extracts address bits available for registers within a block
#define LOCK__BLOCK_MASK 0x3FFF

// Offset into the mirrored address range that locks as a side effect
#define LOCK__OFFSET 0x2000

// Start address of the soft lock bit (SLB) registers
#define LOCK__SLB_OFFSET 0x3800

// Offset of the GCR register from the base address of the block
#define LOCK__GCR_OFFSET 0x3FFC


// Determines the base address of the current block from a given register addr
#define LOCK__BLOCK_BASE(reg_addr) \
   ((reg_addr) & (~LOCK__BLOCK_MASK))

// Determines the address offset of the related SLBR from a given register addr
//  NOTE: there is one 8 bit SLB register per 32 bit module register 
#define LOCK__SLB_REG_OFFSET(reg_addr) \
   (LOCK__SLB_OFFSET + (((reg_addr) & LOCK__BLOCK_MASK) >> 2))

// Calculates the address of the corresponding SLB register for a given module
//  register address
#define LOCK__SLB_REG_ADDR(reg_addr) \
   (LOCK__BLOCK_BASE(reg_addr) + LOCK__SLB_REG_OFFSET(reg_addr))

// Determines the address of the GCR register from the base address of a block
//  -- any register address would work either, but this is just for stability
#define LOCK__GCR_REG_ADDR(base) \
   (LOCK__BLOCK_BASE(base) + LOCK__GCR_OFFSET)


// Masks used to identify a SLB clear or set operation 
#define LOCK__CLR_MASK 0xF0
#define LOCK__SET_MASK 0xFF

// Determines the write enables and soft lock bits for a given 8 bit register
#define LOCK__WRITE_MASK08(reg_addr,mask) \
   ((0x11 & (mask)) << (3 - ((reg_addr) & 3)))

// Determines the write enables and soft lock bits for a given 16 bit register
#define LOCK__WRITE_MASK16(reg_addr,mask) \
   ((0x33 & (mask)) << (2 - ((reg_addr) & 2)))

// Determines the write enables and soft lock bits for a given 32 bit register
#define LOCK__WRITE_MASK32(reg_addr,mask) \
   (mask)