/*******************************************************************************
* NXP Semiconductors
* (c) Copyright 2016 NXP Semiconductors
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by NXP in this matter are performed AS IS and without any 
warranty. CUSTOMER retains the final decision relative to the total design
and functionality of the end product. NXP neither guarantees nor will be held
liable by CUSTOMER for the success of this project.
NXP DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING,
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE ORE ADVISE SUPPLIED 
TO THE PROJECT BY NXP, AND OR NAY PRODUCT RESULTING FROM NXP SERVICES. 
IN NO EVENT SHALL NXP BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
OUT OF THIS AGREEMENT. 
CUSTOMER agrees to hold NXP harmless against any and all claims demands 
or actions by anyone on account of any damage, or injury, whether commercial,
contractual, or tortuous, rising directly or indirectly as a result 
of the advise or assistance supplied CUSTOMER in connection with product, 
services or goods supplied under this Agreement.
********************************************************************************
* File:             main.c
* Owner:            Lukas Zadrapa
* Version:          1.0
* Date:             Sep-15-2016
* Classification:   General Business Information
* Brief:            Using of Register Protection feature
********************************************************************************
********************************************************************************
* Detailed Description:
* Read attached document "How to use Register Protection on MPC5748G.pdf"
* for detailed explanation.
* This example shows how to lock and unlock register MC_ME.RUN_MC[3].R.
* One option is to write directly to memory via pointers, second option is
* to use macros from header file reg_prot.h. 
* ------------------------------------------------------------------------------
* Test HW:         MPC574XG-324DS Rev.A + MPC574XG-MB Rev.C
* MCU:             PPC5748GMMN6A 1N81M
* Fsys:            160 MHz PLL
* Debugger:        Lauterbach Trace32
* Target:          internal_FLASH
*
********************************************************************************
Revision History:
1.0     Sep-15-2016     Lukas Zadrapa    Initial Version
*******************************************************************************/

/*******************************************************************************
* Includes                                                                     
*******************************************************************************/
#include "MPC5748G_310.h"
#include "reg_prot.h"
#include <ppc_ghs.h>
#include "typedefs.h"
#include "INTC_Init.h"

/*******************************************************************************
* Constants and macros
*******************************************************************************/

/*******************************************************************************
* External objects
*******************************************************************************/

/*******************************************************************************
* Global variables
*******************************************************************************/
uint32_t lock = 0;

/*******************************************************************************
* Local functions
*******************************************************************************/

/*******************************************************************************
Function Name : Sys_Init
Engineer      : Lukas Zadrapa
Date          : Sep-07-2016
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : Clock settings - configure 160MHz system clock using
                40MHz crystal
Issues        : NONE
*******************************************************************************/
void Sys_Init(void)
{    
    /* enable all modes, enable all peripherals */
    MC_ME.ME.R        = 0x000005FF;
    MC_ME.RUN_PC[0].R = 0x000000FE;
    /* Connect XOSC to PLL */
    MC_CGM.AC5_SC.B.SELCTL = 1;              
    /* Configure PLL0 Dividers - 160MHz from 40Mhx XOSC */
    PLLDIG.PLLDV.B.PREDIV  = 2;
    PLLDIG.PLLDV.B.MFD     = 32;
    PLLDIG.PLLDV.B.RFDPHI  = 1;
    PLLDIG.PLLCAL3.R       = 0x09C3C000;
    PLLDIG.PLLFD.B.SMDEN   = 1;       /* Sigma delta modulation disabled */
    /* switch to PLL */
    MC_ME.DRUN_MC.R  = 0x00130172;    /* FLASH in normal mode, PLLON, FXOSC ON, Use PLL_PHI_0 */
    
    MC_ME.MCTL.R     = 0x30005AF0;    /* DRUN target mode + KEY */
    MC_ME.MCTL.R     = 0x3000A50F;    /* DRUN target mode + KEY Inverted */
    
    while(MC_ME.GS.B.S_MTRANS == 1); /* Wait for mode transition complete */
}//Sys_Init


/*******************************************************************************
* Global functions
*******************************************************************************/
void main (void)
{
    Sys_Init();
    INTC_Init();
    
    //For detailed explanation, read attached document "How to use Register Protection on MPC5748G.pdf"

    //just write something to RUN3 register to see we can write the register
    //watch the value of this register in memory map at address 0xFFFB803C
    MC_ME.RUN_MC[3].R = 0x001301F0;
    MC_ME.RUN_MC[3].R = 0x00130010;

    // * * * Example 1 - write directly to memory using pointers * * *
    
    //write to mirror address at 0xFFFBA03C and lock the register
    *(unsigned int*)0xFFFBA03C = 0x001301F0;

    //check the lock bits - all lock bits should be set (0x0F)
    lock = *(unsigned char*)0xFFFBB80F;

    //unlock the register
    *(unsigned char*)0xFFFBB80F = 0xF0;
    
    //... and read it again to see the lock bits are cleared (0x00)
    lock = *(unsigned char*)0xFFFBB80F;

    
    // * * * Example 2 - use macros from reg_prot.h * * * 
    
    //just write something to RUN3 register to see we can write the register
    MC_ME.RUN_MC[3].R = 0x001301F0;
    MC_ME.RUN_MC[3].R = 0x00130010;
    
    //get the lock status of this register. It will be zero because protection is not active
    GET_SOFTLOCK(MC_ME.RUN_MC[3].R, lock);
    
    //write new value to register and lock the register for further access
    WRITE_WITH_LOCK32(MC_ME.RUN_MC[3].R,0x001301F0);
    
    //this write will not work because register is protected - exception will be triggered if this command is executed
    //MC_ME.RUN_MC[3].R = 0x00130010;
    
    //get the lock status - it will be 0x0F - all four bytes are locked
    GET_SOFTLOCK(MC_ME.RUN_MC[3].R, lock);
    
    //clear the lock bits
    CLR_SOFTLOCK32(MC_ME.RUN_MC[3].R);
    
    //we can write the register again
    MC_ME.RUN_MC[3].R = 0x00130010;
    
    //get the lock status - it will be zero, protection is not active
    GET_SOFTLOCK(MC_ME.RUN_MC[3].R, lock);
    
    //set lock
    SET_SOFTLOCK32(MC_ME.RUN_MC[3].R);
    
    //try to write the register - it will not work - exception will be triggered if this command is executed
    //MC_ME.RUN_MC[3].R = 0x001301F0;
    
    //and check status...
    GET_SOFTLOCK(MC_ME.RUN_MC[3].R, lock);        
    
    while(1);    
}
